'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactCodemirror = require('react-codemirror2');

var _codemirror = require('codemirror/lib/codemirror');

var _codemirror2 = _interopRequireDefault(_codemirror);

var _parse = require('./parse');

var _parse2 = _interopRequireDefault(_parse);

var _Tooltip = require('./Tooltip');

var _Tooltip2 = _interopRequireDefault(_Tooltip);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const STATUS_IDLE = 'idle';
const STATUS_PLAY = 'play';
const STATUS_PAUSE = 'pause';

const requestTimer = (fn, delay) => {
  if (!delay) {
    fn();
  } else {
    return setTimeout(fn, delay);
  }
};

class Movie extends _react2.default.Component {
  constructor(...args) {
    var _temp;

    return _temp = super(...args), this.editor = null, this.state = {
      status: STATUS_IDLE,
      queue: [],
      actions: [],
      index: 0
    }, this.updateState = data => new Promise(resolve => this.setState(data, resolve)), this.initMovie = () => {
      if (!('preventCursorMovement' in _codemirror2.default.defaults)) {
        _codemirror2.default.defineOption('preventCursorMovement', false, cm => {
          var handler = (cm, event) => cm.getOption('readOnly') && event.preventDefault();
          cm.on('keydown', handler);
          cm.on('mousedown', handler);
        });
      }

      const editor = this.editor;

      const initialValue = editor.getValue() || '';

      let { value, actions } = (0, _parse2.default)(initialValue);

      // normalize line endings
      value = value.replace(/\r?\n/g, '\n');

      const initialPos = value.indexOf('|');

      value = value.replace(/\|/g, '');
      editor.setValue(value);

      if (initialPos !== -1) {
        editor.setCursor(editor.posFromIndex(initialPos));
      }

      this.setState({
        actions
      });
    }, this.toggle = () => {
      if (this.state.status === STATUS_PLAY) {
        this.pause();
      } else {
        this.play();
      }
    }, this.runQueue = async () => {
      const timerObj = this.state.queue[0];
      if (!timerObj) {
        return;
      }

      const { fn, delay } = timerObj;
      requestTimer(fn, delay);

      await this.updateState({
        queue: this.state.queue.slice(1)
      });

      await this.runQueue();
    }, this.play = async () => {
      if (this.state.status === STATUS_PLAY) {
        // already playing
        return;
      }

      if (this.state.status === STATUS_PAUSE) {
        // revert from paused state
        this.editor.focus();

        await this.updateState({
          status: STATUS_PLAY
        });

        await this.runQueue();
        return;
      }

      if (!this.state.actions.length) {
        this.finish();
        return;
      }

      this.editor.execCommand('revert');
      this.editor.setOption('readOnly', true);
      this.editor.focus();

      await this.updateState({
        status: STATUS_PLAY
      });
    }, this.pause = () => {
      if (this.state.status === STATUS_PLAY) {
        this.setState({
          status: STATUS_PAUSE
        });
      }
    }, this.finish = async () => {
      await this.resetState();
      await this.props.onFinish();
    }, this.stop = async () => {
      await this.resetState();
      await this.props.onStop();
    }, this.resetState = () => {
      this.editor.setOption('readOnly', false);

      return new Promise(r => {
        this.setState({
          status: STATUS_IDLE,
          queue: [],
          index: 0
        }, r);
      });
    }, this.requestTimer = (fn, delay) => {
      if (this.state.status !== STATUS_PLAY) {
        // save function call into a queue till next 'play()' call
        this.setState({
          queue: [...this.state.queue, { fn, delay }]
        });
      } else {
        return requestTimer(fn, delay);
      }
    }, this.handleActionChange = index => {
      this.setState({
        index
      });
    }, this.renderAction = () => {
      const { actions, index } = this.state;
      if (actions.length === 0) {
        return null;
      }

      const action = actions[index];
      if (!action) {
        return null;
      }

      if (action.name !== 'tooltip') {
        return null;
      }

      return _react2.default.createElement(_Tooltip2.default, {
        options: action.options,
        editor: this.editor,
        index: index,
        total: actions.length,
        onActionChange: this.handleActionChange,
        onStop: this.stop,
        onFinish: this.finish,
        timer: this.requestTimer,
        customClasses: this.props.tooltipClasses
      });
    }, _temp;
  }

  componentDidMount() {
    this.initMovie();
  }

  componentDidUpdate(prevProps) {
    if (prevProps.value !== this.props.value) {
      this.initMovie();
      return;
    }

    if (this.props.playing) {
      if (!prevProps.playing) {
        this.play();
      }
    } else {
      if (prevProps.playing) {
        this.pause();
      }
    }
  }

  render() {
    const { className, value } = this.props;

    const defaultOptions = {
      mode: 'text/x-java',
      theme: 'seti',
      lineNumbers: true,
      lineWrapping: true,
      readOnly: false,
      autoRefresh: true,
      showCursorWhenSelecting: false
    };

    const options = _extends({}, defaultOptions, this.props.options);

    return _react2.default.createElement(
      'div',
      { className: className },
      this.state.status !== STATUS_IDLE && this.renderAction(),
      _react2.default.createElement(_reactCodemirror.UnControlled, {
        editorDidMount: editor => {
          this.editor = editor;
        },
        value: value,
        options: options
      })
    );
  }

}

exports.default = Movie;